@@ -4,6 +4,10 @@
 
 import logging
 import os
+import pathlib
+from itertools import chain
+from types import MappingProxyType
+from typing import Iterable, Mapping
 
 from okonomiyaki.versions import EnpkgVersion
 from simplesat.constraints import PrettyPackageStringParser, Requirement
@@ -16,6 +20,8 @@ from simplesat.request import Request
 from fusesoc.capi2.coreparser import Core2Parser
 from fusesoc.core import Core
 from fusesoc.librarymanager import LibraryManager
+from fusesoc.lockfile import LockFile, LockFileMode
+from fusesoc.vlnv import Vlnv, compare_relation
 
 logger = logging.getLogger(__name__)
 
@@ -30,9 +36,12 @@ class DependencyError(Exception):
 
 
 class CoreDB:
+    _mapping: Mapping[str, str] = MappingProxyType({})
+
     def __init__(self):
         self._cores = {}
         self._solver_cache = {}
+        self._lockfile = LockFile()
 
     # simplesat doesn't allow ':', '-' or leading '_'
     def _package_name(self, vlnv):
@@ -45,6 +54,7 @@ class CoreDB:
     def _parse_depend(self, depends):
         # FIXME: Handle conflicts
         deps = []
+
         _s = "{} {} {}"
         for d in depends:
             for simple in d.simpleVLNVs():
@@ -83,6 +93,14 @@ class CoreDB:
             found = list([core["core"] for core in self._cores.values()])
         return found
 
+    def load_lockfile(self, filepath: pathlib.Path, disable_store: bool = False):
+        mode = LockFileMode.LOAD if disable_store else LockFileMode.STORE
+        self._lockfile = LockFile.load(filepath, mode)
+
+    def store_lockfile(self, cores):
+        if self._lockfile.update(cores):
+            self._lockfile.store()
+
     def _solver_cache_lookup(self, key):
         if key in self._solver_cache:
             return self._solver_cache[key]
@@ -110,6 +128,120 @@ class CoreDB:
             h ^= hash(pair)
         return h
 
+    def _lockfile_replace(self, core: Vlnv):
+        """Try to pin the core version from cores defined in the lock file"""
+        cores = self._lockfile.cores_vlnv()
+        if cores:
+            for locked_core in cores:
+                if locked_core.vln_str() == core.vln_str():
+                    valid_version = compare_relation(locked_core, core.relation, core)
+                    if valid_version:
+                        core.version = locked_core.version
+                        core.revision = locked_core.revision
+                        core.relation = "=="
+                    else:
+                        # Invalid version in lockfile
+                        logger.warning(
+                            "Failed to pin core {} outside of dependency version {} {} {}".format(
+                                str(locked_core),
+                                core.vln_str(),
+                                core.relation,
+                                core.version,
+                            )
+                        )
+
+    def _mapping_apply(self, core: Vlnv):
+        """If the core matches a mapping, apply the mapping, mutating the given core."""
+        remapping = self._mapping.get(core.vln_str())
+
+        if not remapping:
+            return
+
+        previous_vlnv = str(core)
+        (core.vendor, core.library, core.name) = remapping.split(":")
+
+        logger.info(f"Mapped {previous_vlnv} to {core}.")
+
+    def mapping_set(self, mapping_vlnvs: Iterable[str]) -> None:
+        """Construct a mapping from the given cores' mappings.
+
+        Takes the VLNV strings of the cores' mappings to apply.
+        Verifies the mappings and applies them.
+        """
+        if self._mapping:
+            raise RuntimeError(
+                "Due to implementation details, mappings can only be applied once."
+            )
+
+        mappings = {}
+        for mapping_vlnv in mapping_vlnvs:
+            new_mapping_name = str(Vlnv(mapping_vlnv))
+            new_mapping_core = self._cores.get(new_mapping_name)
+            if not new_mapping_core:
+                raise RuntimeError(f"The core '{mapping_vlnv}' wasn't found.")
+
+            new_mapping_raw = new_mapping_core["core"].mapping
+            if not new_mapping_raw:
+                raise RuntimeError(
+                    f"The core '{mapping_vlnv}' doesn't contain a mapping."
+                )
+
+            have_versions = list(
+                filter(
+                    lambda vlnv: (vlnv.relation, vlnv.version) != (">=", "0"),
+                    map(Vlnv, chain(new_mapping_raw.keys(), new_mapping_raw.values())),
+                )
+            )
+            if have_versions:
+                raise RuntimeError(
+                    "Versions cannot be given as part of a mapping."
+                    f"\nThe mapping of {mapping_vlnv} following has"
+                    f" the following version constraints:\n\t{have_versions}"
+                )
+
+            new_mapping = {
+                Vlnv(source).vln_str(): Vlnv(destination).vln_str()
+                for source, destination in new_mapping_raw.items()
+            }
+            new_src_set = new_mapping.keys()
+            new_dest_set = frozenset(new_mapping.values())
+            curr_src_set = mappings.keys()
+            curr_dest_set = frozenset(mappings.values())
+
+            new_src_dest_overlap = new_mapping.keys() & new_dest_set
+            if new_src_dest_overlap:
+                raise RuntimeError(
+                    "Recursive mappings are not supported."
+                    f"\nThe mapping {mapping_vlnv} has the following VLNV's"
+                    f" in both it's sources and destinations:\n\t{new_src_dest_overlap}."
+                )
+
+            source_overlap = curr_src_set & new_src_set
+            if source_overlap:
+                raise RuntimeError(
+                    f"The following sources are in multiple mappings:\n\t{source_overlap}."
+                )
+
+            dest_overlap = new_dest_set & curr_dest_set
+            if dest_overlap:
+                raise RuntimeError(
+                    f"The following destinations are in multiple mappings:\n\t{dest_overlap}."
+                )
+
+            src_dest_overlap = (new_src_set | curr_src_set) & (
+                new_dest_set | curr_dest_set
+            )
+            if src_dest_overlap:
+                raise RuntimeError(
+                    "Recursive mappings are not supported."
+                    f"\nThe following VLNV's are in both the sources and"
+                    f" destinations:\n\t{src_dest_overlap}."
+                )
+
+            mappings.update(new_mapping)
+
+        self._mapping = MappingProxyType(mappings)
+
     def solve(self, top_core, flags):
         return self._solve(top_core, flags)
 
@@ -195,8 +327,13 @@ class CoreDB:
                 _flags["is_toplevel"] = core.name == top_core
                 _depends = core.get_depends(_flags)
                 if _depends:
+                    for depend in _depends:
+                        self._mapping_apply(depend)
+                        self._lockfile_replace(depend)
                     _s = "; depends ( {} )"
                     package_str += _s.format(self._parse_depend(_depends))
+            else:
+                self._lockfile_replace(top_core)
 
             parser = PrettyPackageStringParser(EnpkgVersion.from_string)
 
@@ -226,6 +363,7 @@ class CoreDB:
             raise DependencyError(top_core.name)
 
         virtual_selection = {}
+        partial_lockfile = False
         objdict = {}
         if len(transaction.operations) > 1:
             for op in transaction.operations:
@@ -244,6 +382,8 @@ class CoreDB:
                     if p[0] in virtual_selection:
                         # If package that implements a virtual core is required, remove from the dictionary
                         del virtual_selection[p[0]]
+                if not self._lockfile.core_vlnv_exists(op.package.core.name):
+                    partial_lockfile = True
                 op.package.core.direct_deps = [
                     objdict[n[0]] for n in op.package.install_requires
                 ]
@@ -254,6 +394,8 @@ class CoreDB:
                     virtual[1], virtual[0]
                 )
             )
+        if partial_lockfile and not self._lockfile.no_cores():
+            logger.warning("Using lock file with partial list of cores")
 
         result = [op.package.core for op in transaction.operations]
 
@@ -286,7 +428,7 @@ class CoreManager:
         visited = set()
         for root, dirs, files in os.walk(path, followlinks=True):
             ignore_tree = ("FUSESOC_IGNORE" in files) or (
-                os.path.abspath(root) in ignored_dirs
+                os.path.realpath(root) in ignored_dirs
             )
             if ignore_tree:
                 del dirs[:]
