@@ -91,9 +91,14 @@ class Edalizer:
                 logger.info(f"Applying filter {f}")
                 self.edam = filter_class().run(self.edam, self.work_root)
             except ModuleNotFoundError:
-                logger.warning(f"Ignoring unknown EDAM filter '{f}'")
+                raise RuntimeError(f"Could not find EDAM filter '{f}'")
             except Exception as e:
-                logger.error(f"Filter error: {str(e)}")
+                import traceback
+
+                traceback.print_exc()
+                raise RuntimeError(f"Filter error: {str(e)}")
+            except SystemExit as e:
+                raise RuntimeError(f"Filter exited with error code {str(e)}")
 
     def run(self):
         """Run all steps to create a EDAM file"""
@@ -111,6 +116,9 @@ class Edalizer:
         # Create EDAM file contents
         self.create_edam()
 
+        # Write lockfile if appropriate
+        self.core_manager.db.store_lockfile(self.cores)
+
         return self.edam
 
     def _core_flags(self, core):
@@ -219,6 +227,15 @@ class Edalizer:
 
             rel_root = os.path.relpath(files_root, self.work_root)
 
+            # Extract core description file
+            snippet["cores"] = {
+                str(core.name): {
+                    "core_file": os.path.relpath(core.core_file, self.work_root),
+                    "dependencies": core.direct_deps,
+                    "license": core.get_license(),
+                }
+            }
+
             # Extract parameters
             snippet["parameters"] = core.get_parameters(_flags, parameters)
             merge_dict(parameters, snippet["parameters"])
@@ -242,9 +259,14 @@ class Edalizer:
             for file in core.get_files(_flags):
 
                 # Reparent file path
-                file["name"] = str(
-                    file.get("copyto", os.path.join(rel_root, file["name"]))
-                )
+                if "copyto" in file:
+                    _cpt = pathlib.Path(file["copyto"])
+                    if (self.work_root / _cpt).is_dir():
+                        file["name"] = str(_cpt / pathlib.Path(file["name"]).name)
+                    else:
+                        file["name"] = file["copyto"]
+                else:
+                    file["name"] = str(os.path.join(rel_root, file["name"]))
 
                 # Set owning core
                 file["core"] = str(core.name)
@@ -544,9 +566,10 @@ class Ttptttg:
         }
 
     def _sha256_input_yaml_hexdigest(self):
-        return hashlib.sha256(
-            utils.yaml_dump(self.generator_input).encode()
-        ).hexdigest()
+        data = self.generator_input.copy()
+        # Remove files_root since that is not deterministic
+        data.pop("files_root")
+        return hashlib.sha256(utils.yaml_dump(data).encode()).hexdigest()
 
     def _sha256_file_input_hexdigest(self):
         input_files = []
@@ -580,17 +603,6 @@ class Ttptttg:
 
         return hash.hexdigest()
 
-    def _fwrite_hash(self, hashfile, data):
-        with open(hashfile, "w") as f:
-            f.write(data)
-
-    def _fread_hash(self, hashfile):
-        data = ""
-        with open(hashfile) as f:
-            data = f.read()
-
-        return data
-
     def _run(self, generator_cwd):
         logger.info("Generating " + str(self.vlnv))
 
@@ -632,6 +644,11 @@ class Ttptttg:
     def is_cacheable(self):
         return self.is_input_cacheable() or self.is_generator_cacheable()
 
+    def acquire_cache_lock(self):
+        have_lock = False
+        # while not have_lock:
+        #    if
+
     def generate(self):
         """Run a parametrized generator
 
@@ -641,7 +658,7 @@ class Ttptttg:
 
         hexdigest = self._sha256_input_yaml_hexdigest()
 
-        logger.debug("Generator input yaml hash: " + hexdigest)
+        logger.debug("Generator parameters hash: " + hexdigest)
 
         generator_cwd = os.path.join(
             self.gen_root,
@@ -649,6 +666,16 @@ class Ttptttg:
             self.vlnv.sanitized_name + "-" + hexdigest,
         )
 
+        if "file_input_parameters" in self.generator:
+            # If file_input_parameters has been configured in the generator
+            # parameters will be iterated to look for files to add to the
+            # input files hash calculation.
+            file_input_hash = self._sha256_file_input_hexdigest()
+            generator_cwd = os.path.join(generator_cwd, file_input_hash)
+            logger.debug("Generator input files hash: " + file_input_hash)
+
+        logger.debug("Generator cwd: " + generator_cwd)
+
         if os.path.lexists(generator_cwd) and not os.path.isdir(generator_cwd):
             raise RuntimeError(
                 "Unable to create generator working directory since it already exists and is not a directory: "
@@ -657,59 +684,28 @@ class Ttptttg:
                 + "Remove it manually or run 'fusesoc gen clean'"
             )
 
-        if self.is_input_cacheable():
-            # Input cache enabled. Check if cached output already exists in generator_cwd.
-            logger.debug("Input cache enabled.")
-
-            # If file_input_parameters has been configured in the generator
-            # parameters will be iterated to look for files to add to the
-            # input files hash calculation.
-            if "file_input_parameters" in self.generator:
-                file_input_hash = self._sha256_file_input_hexdigest()
-
-                logger.debug("Generator file input hash: " + file_input_hash)
-
-                hashfile = os.path.join(generator_cwd, ".fusesoc_file_input_hash")
-
-                rerun = False
-
-                if os.path.isfile(hashfile):
-                    cached_hash = self._fread_hash(hashfile)
-                    logger.debug("Cached file input hash: " + cached_hash)
-
-                    if not file_input_hash == cached_hash:
-                        logger.debug("File input has changed.")
-                        rerun = True
-                    else:
-                        logger.info("Found cached output for " + str(self.vlnv))
-
-                else:
-                    logger.debug("File input hash file does not exist: " + hashfile)
-                    rerun = True
-
-                if rerun:
-                    shutil.rmtree(generator_cwd, ignore_errors=True)
-                    self._run(generator_cwd)
-                    self._fwrite_hash(hashfile, file_input_hash)
-
-            elif os.path.isdir(generator_cwd):
-                logger.info("Found cached output for " + str(self.vlnv))
-            else:
-                # No directory found. Run generator.
-                self._run(generator_cwd)
-
-        elif self.is_generator_cacheable():
-            # Generator cache enabled. Call the generator and let it
-            # decide if the old output still is valid.
-            logger.debug("Generator cache enabled.")
-            self._run(generator_cwd)
+        if self.is_input_cacheable() and os.path.isdir(generator_cwd):
+            logger.info("Found cached output for " + str(self.vlnv))
         else:
-            # No caching enabled. Try to remove directory if it already exists.
-            # This could happen if a generator that has been configured with
-            # caching is changed to no caching.
-            logger.debug("Generator cache is not enabled.")
-            shutil.rmtree(generator_cwd, ignore_errors=True)
-            self._run(generator_cwd)
+            # TODO: Acquire a lock here to ensure that we are the only users
+            if self.is_generator_cacheable():
+                logger.warning(
+                    "Support for generator-side cachable cores are deprecated and will be removed"
+                )
+            else:
+                shutil.rmtree(generator_cwd, ignore_errors=True)
+            try:
+                self._run(generator_cwd)
+            except:
+                # If the generator invocation failed for any reason, its output
+                # directory is removed. While this is bad for debugging failing
+                # generators, it at least prevents the next FuseSoC-run to
+                # assume, that the generator was successful and hence a failed
+                # generator is retried on the next FuseSoC-run.
+                logger.debug("Generator failed, removing its output files")
+                shutil.rmtree(generator_cwd, ignore_errors=False)
+                raise
+            # TODO: Release cache lock
 
         cores = []
         logger.debug("Looking for generated or cached cores in " + generator_cwd)
