@@ -1,16 +1,22 @@
 #!/usr/bin/env python
+# PYTHON_ARGCOMPLETE_OK
 # Copyright FuseSoC contributors
 # Licensed under the 2-Clause BSD License, see LICENSE for details.
 # SPDX-License-Identifier: BSD-2-Clause
 
 import argparse
 import os
+import pathlib
 import shutil
 import signal
 import sys
 import warnings
 from pathlib import Path
 
+import argcomplete
+
+from fusesoc import signature
+
 try:
     from fusesoc.version import version as __version__
 except ImportError:
@@ -183,6 +189,15 @@ def library_list(fs, args):
 
 def list_cores(fs, args):
     cores = fs.get_cores()
+    trustfile = fs.config.ssh_trustfile or args.ssh_trustfile
+    if not trustfile:
+        logger.warn(
+            "No trustfile configured (ssh-trustfile in fusesoc.conf), signatures will not be checked."
+        )
+    elif not os.path.isfile(trustfile):
+        logger.warn(
+            "The trustfile configured in fusesoc.conf does not exist, signatures will not be checked."
+        )
     print("\nAvailable cores:\n")
     if not cores:
         cores_root = fs.get_libraries()
@@ -192,7 +207,7 @@ def list_cores(fs, args):
             logger.error("No libraries registered")
         exit(1)
     maxlen = max(map(len, cores.keys()))
-    print("Core".ljust(maxlen) + "  Cache status  Description")
+    print("Core".ljust(maxlen) + "  Cache status  Signature  Description")
     print("=" * 80)
     for name in sorted(cores.keys()):
         core = cores[name]
@@ -201,6 +216,8 @@ def list_cores(fs, args):
             + " : "
             + core.cache_status().rjust(10)
             + " : "
+            + core.sig_status(trustfile).rjust(8)
+            + " : "
             + (core.get_description() or "<No description>")
         )
 
@@ -236,7 +253,7 @@ def gen_list(fs, args):
                     "{} : {} : {}".format(
                         core.ljust(maxlen),
                         generator_name,
-                        generator_data["description"] or "<No description>",
+                        generator_data.get("description", "<No description>"),
                     )
                 )
 
@@ -264,7 +281,21 @@ Usage       :
 
 def core_info(fs, args):
     core = _get_core(fs, args.core)
-    print(core.info())
+    trustfile = fs.config.ssh_trustfile or args.ssh_trustfile
+    print(core.info(trustfile))
+
+
+def core_sign(fs, args):
+    core = _get_core(fs, args.core)
+    logger.info("sign core file: " + core.core_file)
+    logger.info("with key file: " + args.keyfile)
+    sigfile = core.core_file + ".sig"
+    logger.info("put result in: " + sigfile)
+    sig = signature.sign(core, args.keyfile, None)
+    file = open(sigfile, "w")
+    file.write(sig)
+    file.close()
+    print(f"{sigfile} created")
 
 
 def gen_clean(fs, args):
@@ -303,6 +334,15 @@ def run(fs, args):
         else:
             flags[flag] = True
 
+    fs.cm.db.mapping_set(args.mapping)
+
+    if args.lockfile is not None:
+        try:
+            fs.cm.db.load_lockfile(args.lockfile)
+        except SyntaxError as e:
+            logger.error(f"Failed to load lock file, {str(e)}")
+            exit(1)
+
     core = _get_core(fs, args.system)
 
     try:
@@ -315,8 +355,8 @@ def run(fs, args):
         exit(1)
 
     # Unconditionally clean out the work root on fresh builds
-    # if we use the old tool API
-    if do_configure and not core.get_flow(flags):
+    # if we use the old tool API or clean flag is set
+    if do_configure and (not core.get_flow(flags) or args.clean):
         try:
             prepare_work_root(fs.get_work_root(core, flags))
         except RuntimeError as e:
@@ -397,6 +437,41 @@ def update(fs, args):
     fs.update_libraries(args.libraries)
 
 
+class CoreCompleter:
+    def __call__(self, parsed_args, **kwargs):
+        config = Config(parsed_args.config)
+        args_to_config(parsed_args, config)
+        fs = Fusesoc(config)
+        cores = fs.get_cores()
+        return cores
+
+
+class ToolCompleter:
+    def __call__(self, parsed_args, **kwargs):
+        from edalize.edatool import get_edatool, walk_tool_packages
+
+        _tp = list(walk_tool_packages())
+        tools = []
+        for tool_name in _tp:
+            try:
+                tool_class = get_edatool(tool_name)
+                if tool_class.get_doc(0)["description"]:
+                    tools += [tool_name]
+            # Ignore any misbehaving backends
+            except Exception:
+                pass
+        return tools
+
+
+class GenCompleter:
+    def __call__(self, parsed_args, **kwargs):
+        config = Config(parsed_args.config)
+        args_to_config(parsed_args, config)
+        fs = Fusesoc(config)
+        cores = fs.get_generators()
+        return cores
+
+
 def get_parser():
     parser = argparse.ArgumentParser()
     subparsers = parser.add_subparsers()
@@ -425,6 +500,7 @@ def get_parser():
     )
     parser.add_argument("--verbose", help="More info messages", action="store_true")
     parser.add_argument("--log-file", help="Write log messages to file")
+    parser.add_argument("--ssh-trustfile", help="Override trustfile in fusesoc.conf")
 
     # fetch subparser
     parser_fetch = subparsers.add_parser(
@@ -448,9 +524,20 @@ def get_parser():
     parser_core_show = core_subparsers.add_parser(
         "show", help="Show information about a core"
     )
-    parser_core_show.add_argument("core", help="Name of the core to show")
+    parser_core_show.add_argument(
+        "core", help="Name of the core to show"
+    ).completer = CoreCompleter()
     parser_core_show.set_defaults(func=core_info)
 
+    parser_core_sign = core_subparsers.add_parser(
+        "sign", help="Create user signature for a core"
+    )
+    parser_core_sign.add_argument(
+        "core", help="Name of the core to sign"
+    ).completer = CoreCompleter()
+    parser_core_sign.add_argument("keyfile", help="File containing ssh private key")
+    parser_core_sign.set_defaults(func=core_sign)
+
     # tool subparser
     parser_tool = subparsers.add_parser(
         "tool", help="Subcommands for dealing with tools"
@@ -470,7 +557,7 @@ def get_parser():
     parser_core_info = subparsers.add_parser(
         "core-info", help="Display details about a core"
     )
-    parser_core_info.add_argument("core")
+    parser_core_info.add_argument("core").completer = CoreCompleter()
     parser_core_info.set_defaults(func=core_info)
 
     # gen subparser
@@ -490,7 +577,9 @@ def get_parser():
     parser_gen_show = gen_subparsers.add_parser(
         "show", help="Show information about a generator"
     )
-    parser_gen_show.add_argument("generator", help="Name of the generator to show")
+    parser_gen_show.add_argument(
+        "generator", help="Name of the generator to show"
+    ).completer = GenCompleter()
     parser_gen_show.set_defaults(func=gen_show)
 
     # gen clean subparser
@@ -536,7 +625,7 @@ def get_parser():
     parser_library_add.add_argument(
         "--sync-type",
         help="The provider type for the library. Defaults to 'git'.",
-        choices=["git", "local"],
+        choices=["git", "local", "url"],
         dest="sync-type",
     )
     parser_library_add.add_argument(
@@ -568,6 +657,11 @@ def get_parser():
 
     # run subparser
     parser_run = subparsers.add_parser("run", help="Start a tool flow")
+    parser_run.add_argument(
+        "--clean",
+        action="store_true",
+        help="Clean build directory on start",
+    )
     parser_run.add_argument(
         "--no-export",
         action="store_true",
@@ -591,7 +685,9 @@ def get_parser():
     parser_run.add_argument("--build", action="store_true", help="Execute build stage")
     parser_run.add_argument("--run", action="store_true", help="Execute run stage")
     parser_run.add_argument("--target", help="Override default target")
-    parser_run.add_argument("--tool", help="Override default tool for target")
+    parser_run.add_argument(
+        "--tool", help="Override default tool for target"
+    ).completer = ToolCompleter()
     parser_run.add_argument(
         "--flag",
         help="Set custom use flags. Can be specified multiple times",
@@ -611,10 +707,23 @@ def get_parser():
         action="store_true",
         help="Allow additional properties in core files",
     )
-    parser_run.add_argument("system", help="Select a system to operate on")
+    parser_run.add_argument(
+        "system", help="Select a system to operate on"
+    ).completer = CoreCompleter()
     parser_run.add_argument(
         "backendargs", nargs=argparse.REMAINDER, help="arguments to be sent to backend"
     )
+    parser_run.add_argument(
+        "--lockfile",
+        help="Lockfile file path",
+        type=pathlib.Path,
+    )
+    parser_run.add_argument(
+        "--mapping",
+        help="The VLNV of a core's mapping to apply.",
+        default=[],
+        action="append",
+    )
     parser_run.set_defaults(func=run)
 
     # config subparser
@@ -634,6 +743,7 @@ def get_parser():
 def parse_args(argv):
     parser = get_parser()
 
+    argcomplete.autocomplete(parser, always_complete_options=False)
     args = parser.parse_args(argv)
 
     if hasattr(args, "func"):
