@@ -324,11 +324,13 @@ def test_capi2_get_generators():
     core = Core(Core2Parser(), os.path.join(cores_dir, "generate", "generators.core"))
 
     generators = core.get_generators()
-    assert len(generators) == 2
+    assert len(generators) == 3
     assert generators["generator1"]["command"] == "testgen.py"
     assert generators["generator2"]["command"] == "testgen.py"
     assert generators["generator2"]["cache_type"] == "input"
-    assert generators["generator2"]["file_input_parameters"] == "file_in_param1"
+    assert generators["generator3"]["command"] == "testgen.py"
+    assert generators["generator3"]["cache_type"] == "input"
+    assert generators["generator3"]["file_input_parameters"] == "file_in_param1"
 
 
 def test_capi2_get_parameters():
@@ -606,6 +608,12 @@ def test_capi2_get_ttptttg():
             "name": "testgenerate_with_cache",
             "generator": "generator2",
             "pos": "append",
+            "config": {"some_option": "some_value"},
+        },
+        {
+            "name": "testgenerate_with_file_cache",
+            "generator": "generator3",
+            "pos": "append",
             "config": {"file_in_param1": "file_cachetest"},
         },
     ]
@@ -663,8 +671,8 @@ def test_capi2_info():
         gen_info = "\n".join(
             [
                 x
-                for x in core.info().split("\n")
-                if not any([y in x for y in ["Core root", "Core file"]])
+                for x in core.info(None).split("\n")
+                if not any([y in x for y in ["Core root", "Core file", "Signature"]])
             ]
         )
         with open(os.path.join(tests_dir, __name__, core_name + ".info")) as f:
